
# q-expansion of newform 1560.4.a.t, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1560
weight = 4
poly_data = [-2107216, -1167798, 172073, 15963, -1422, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[-129452261, 71893199, 2163569, -947750, 11672, 866], 7926255], [[-4436776588, -108567308, 69649402, 1977710, -115919, -3017], 23778765], [[-1506832166, 326129426, 40471409, 2647942, -114205, -3916], 23778765], [[76365605, 372648259, -23242790, -4141906, 94036, 4321], 23778765], [[-545261537, -203132404, 28985663, 3643918, -100759, -4186], 7926255]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0, 0, 0], [3, 0, 0, 0, 0, 0], [-5, 0, 0, 0, 0, 0], [-1, 1, 0, 0, 0, 0], [-1, 0, 0, 0, 1, 0], [-13, 0, 0, 0, 0, 0], [15, 1, 0, 0, 1, 1], [-3, 1, 0, 1, 0, 0], [2, 2, -1, 0, 0, 0], [20, -2, 1, -1, -2, 0], [-18, 3, 1, -1, 2, -1], [-52, -5, 1, 0, -3, -2], [46, 3, -3, 0, 1, 0], [125, 9, 1, 0, -1, -1], [60, 1, -1, -1, 0, -3], [65, -4, 6, -2, 0, 1], [139, 2, 2, 1, -1, 2], [30, -2, -2, -1, -10, -2], [247, 8, -2, 1, -5, 0], [65, 2, 0, -2, 9, 0], [109, -7, -2, 3, 5, 5], [303, 19, -1, 1, -3, 7], [179, -6, 0, 3, 9, 0], [1, -15, 1, 1, -3, 1], [251, -32, 5, -5, 10, -5], [-124, -23, -2, 4, 1, 8], [644, 26, -2, 2, -2, 4], [508, -63, 5, -2, 7, -4], [403, 19, 8, -3, 1, 1], [491, -23, 6, -7, 1, -5], [1210, -11, 1, -5, 3, 4], [733, -20, 3, 2, 6, -7], [475, -10, -12, 9, -6, 1], [910, 23, -7, 0, 3, -12], [-402, 55, 5, -10, -19, -20], [452, 34, 0, 0, -15, -7], [1239, 24, 2, -1, 6, 15], [149, 6, 6, 18, -5, -6], [320, -42, 2, 6, -5, -13], [-839, -4, -14, -3, 12, 7], [505, -25, 20, -7, -3, -3], [1218, -8, -10, 19, -2, -16], [386, 39, 11, 7, 27, 22], [1527, 26, 13, -7, 20, -1], [-303, 53, 19, -18, 21, 19], [-471, -12, -4, 1, -26, 3], [648, 38, -18, -10, 24, -6], [-791, 87, 24, -9, -52, -26], [-334, -6, -8, -12, 39, -23], [-249, -14, 15, 12, -22, 23], [261, 15, -2, 14, -63, -7], [-1020, 6, -40, 25, 27, 25], [2676, -21, 7, -13, 13, 0], [-671, -67, -14, -3, 35, 33], [-183, -41, -4, -31, 13, -13], [-1050, 57, 16, -24, 19, -6], [90, 23, -32, 0, 29, -8], [-700, 134, -8, 2, -19, -35], [-461, -233, 1, 20, -67, -29], [1965, 56, -24, 19, -78, 5], [-1161, -82, -18, 5, 54, -9], [-59, -88, 46, -1, 30, 1], [287, -46, -40, 30, -81, -8], [24, 123, 11, 5, -19, 22], [3792, 120, -30, -8, -22, -8], [300, 155, 15, -15, 69, -20], [-1471, 72, 39, -32, 21, 58], [3788, -147, 19, -23, -75, -14], [1562, -97, -7, 40, 21, -34], [1387, -96, -59, 16, 120, 63], [781, -119, -17, 28, 43, 21], [-1129, 14, 28, -33, -45, -6], [-568, 63, -13, 53, -41, 20], [3395, -277, 33, -46, -105, 17], [-2697, -164, -11, -2, -131, -36], [-3800, 286, -22, 8, 53, 5], [-3006, -95, 2, 42, -25, 56], [5824, -27, -57, 14, 59, 78], [3586, -296, 22, -26, 12, -16], [1818, 285, 27, -7, 55, 2], [-4169, -75, 40, 11, -59, -11], [2953, -149, 42, -19, -159, -85], [-274, 344, 6, -36, 93, 13], [168, -193, 17, 19, -11, -32], [-3614, -231, 41, 20, -5, -54], [-304, 239, 5, -2, -115, 4], [-121, -189, 35, -41, 23, 27], [3431, 0, -11, 11, -146, -109], [1184, 77, -19, 22, -49, 34], [-867, -7, -26, -52, 12, 30], [-2660, 269, -23, 2, 65, 38], [-6326, -20, 18, -33, -33, -3], [-4505, -9, -14, 12, 40, -86], [-837, -234, 13, -36, -38, -47], [-3141, 27, -44, -33, -226, 22], [3764, -357, 34, -16, 5, 80], [-3746, -199, 49, -14, 215, 16], [5776, 216, -14, 4, -34, -36], [3260, 61, -83, 43, 123, 18], [-1015, 223, 30, 39, 289, -45], [4894, 0, -112, 2, 192, 42], [-3720, 335, 43, -109, 133, 40], [-3554, -71, 139, -6, 61, -84], [-3984, -147, -25, -35, -329, 28], [-4530, 99, 37, 6, 225, -90], [1627, 306, 1, -25, 176, 83], [-5018, 340, 54, -6, -95, 69], [-6979, -8, 20, -143, -4, 43], [-6216, -8, -148, 40, 78, -38], [7825, 220, 86, 4, -36, 17], [232, -445, 71, -51, -45, 44], [2016, -59, -73, 68, -17, 54], [205, -182, 52, -93, -202, 19], [-4173, -374, -5, 28, -159, 98], [-4780, -214, 38, 10, -203, 9], [-4148, -459, -35, 7, 189, 118], [-10497, 286, 12, 78, 141, -122], [-3576, -392, -101, 44, 66, 10], [-2259, -694, -22, -47, 40, -23], [-4757, -119, 112, -59, -171, -55], [7513, 416, -117, 52, -256, 1], [4446, -397, 67, 5, -117, -44], [-3943, 214, 106, 30, -82, -23], [1885, -345, -43, 120, 18, -18], [-6449, -123, -16, -5, -232, -66], [-4740, 45, 26, -12, -141, -142], [847, -571, 106, -65, -65, 49], [-12708, 252, 26, -18, 160, -28], [-4504, -474, -120, 106, -26, 108], [-3926, 801, -41, 84, 11, 124], [-9691, -119, -38, 67, -218, 30], [-3330, 430, -126, 74, 221, -53], [-9534, 269, -91, -28, 115, 46], [9320, -420, 20, -96, -34, -72], [366, 350, 100, -88, 294, -120], [-7764, -75, 101, 47, 279, -20], [1631, 110, -52, 43, -196, 155], [9821, -58, 150, 43, 151, 20], [-3945, 710, -184, 88, -168, 141], [-3288, -88, -168, 18, 18, 20], [-7179, -92, -121, -124, -307, -30], [-4796, -227, -65, -88, 63, -94], [4708, 506, 14, 114, 170, 96], [-11511, 1, 155, -84, 236, -162], [-12280, -108, 52, 110, 256, 118], [-6896, 230, 232, -139, -19, -29], [-15062, -303, -145, 14, -381, -72], [-7347, 149, 150, 58, -173, 233], [2434, -19, 113, 28, -247, -12], [5928, -524, 84, 58, -197, -67], [3326, -906, 86, -154, -118, 32], [-7902, -575, 107, -23, 87, -122], [10524, 398, 34, 102, 68, 10], [3372, -416, 165, -10, -368, 28], [8271, 788, -28, -39, 584, 87], [6722, 591, -143, 45, 347, 82], [-2146, 163, -185, 48, 103, -184], [-19391, -793, -165, -25, -193, -283], [16282, -661, -141, 65, 41, -102], [1514, -111, 55, 32, -305, 232], [-3175, -300, -226, 123, -327, 152], [22383, 477, -128, -24, -53, -21], [-4581, -35, 128, 221, -74, -8], [7711, 788, 57, 156, 182, 11], [-1203, -777, -41, -106, -347, -303], [19364, 594, -222, -74, 75, 139], [-5203, -77, 143, -23, 33, -115], [-4021, -1322, -104, 103, -590, -151], [-11686, -803, -103, -121, -137, -74], [-12095, -384, 138, -194, 360, 5], [-829, 220, 22, -147, 199, -88], [8959, 379, -72, -21, -663, -255], [20078, 1001, -37, -1, 163, 216], [1213, -846, 45, 21, -640, -363], [-1865, 1131, 7, 39, 339, 21], [-13336, -247, 319, -151, 35, 276], [-5469, -1156, 143, -120, 271, -222], [4256, 811, -51, -80, -227, 226], [17722, 460, -282, -4, -424, 62], [-18520, -88, -162, -72, 166, 168], [4157, -1601, 98, 8, -362, -130], [-24219, -154, -87, 122, -178, -193], [19756, -19, -319, 101, 33, 310], [-6483, 265, 77, -198, 247, 277], [20994, -1058, 118, -162, -157, -255], [-9700, -279, 164, 82, 531, 194], [16078, -1017, -141, 39, 149, 386], [19897, -592, 94, 0, -573, -80], [7674, 317, 247, 155, -41, -178], [18036, 776, -374, 13, -281, -349], [14188, -339, 141, -209, 57, -38], [21096, -850, 116, 78, 227, 215], [7322, -1807, -27, 98, -139, -72], [198, 911, 253, -76, -131, -24], [-3002, 520, -174, -116, -362, -160], [16063, 517, 198, 58, -141, 127], [12030, -150, -300, 296, -358, -24], [-3955, 633, 271, 152, 173, -129], [-24877, 586, -106, 321, 170, 117], [37368, -414, -239, 208, 90, -80], [-1314, -392, 280, 128, -624, -188], [1084, 809, 165, -276, 69, -298], [1722, 79, 259, 58, -159, 452], [11499, -436, 174, 36, -408, -45], [-2125, -162, 98, -24, -689, -448], [-12587, -162, -2, -88, -466, -423], [-33890, 602, -208, 172, -57, 137], [47880, 582, -156, 108, -406, -60], [-9106, -2045, -59, -154, 101, -200], [-12900, -1595, 161, 54, 27, 340], [23918, 1550, -288, -14, 38, 204], [32850, -342, -267, -7, -312, 440], [52380, 586, 52, 116, 136, 246], [16647, -923, -189, 258, 420, 24], [5042, -1118, 56, 74, -1033, -167], [-6452, -535, 153, -201, -689, -520], [9084, 521, -313, -175, 561, -4], [-2312, 1311, -27, -139, 421, 268], [18260, -584, 28, 122, 263, 497], [-21252, 292, 140, 242, -156, 50], [11398, -448, 434, 57, 616, -154], [-23672, -1586, -286, -260, -567, 65], [-20944, -1066, -216, -58, 18, 278], [1285, 795, 46, 149, 292, 34], [-20290, -547, -9, 46, -687, 148], [14128, -382, -180, -353, -26, 64], [25735, -1461, -102, -172, 133, -125], [27022, -1131, 223, -395, -155, -266], [13714, 1644, 330, 114, -592, -318], [5751, 1262, 58, 114, -1, -360], [-30655, -231, 79, -206, 741, -209], [-12490, 527, 175, 294, 495, 380], [-16468, -1143, -165, 25, -32, -271], [10130, 606, -192, -362, 416, -18], [7955, -502, 122, -13, -1560, -79], [7552, 603, -165, 307, 848, -153], [3866, -559, -185, 131, -601, 274], [7916, -1844, 144, -174, 188, 138], [45723, -231, 410, 37, 35, -61], [37110, 753, -425, -277, 777, 66], [311, 168, 266, -271, 287, -108], [-27122, -1467, 81, 92, -1213, 222], [49560, 78, 180, -44, 516, 146], [-16551, 86, -595, -38, -634, -143], [18815, -2741, -189, 56, 509, -33], [-5412, -983, -123, 187, 287, 454], [30103, 595, -92, -247, -30, -590], [-19987, 176, -286, 124, -423, -158], [-65657, -1213, -322, 47, -398, -306], [24910, -297, -230, 478, 511, -124], [17866, -792, 362, -154, -1408, -420], [11997, -448, 92, -61, -378, -535], [-11406, -1003, -117, -415, 356, -453], [-28114, -826, 202, 107, 180, 428], [-20309, -990, -100, -292, -1164, -199], [44235, -1018, -260, 178, -613, 632], [-14148, 24, 644, -238, 938, -28], [-28601, -1736, -204, -110, -337, 10], [3951, 782, -60, 65, -1048, -61], [51305, -1092, 89, 53, 54, -127], [9386, 138, -372, -52, -102, 30], [10508, 47, 335, 154, 1401, 68], [-53477, 1405, 362, -281, -281, -715], [-10814, 1193, -455, 125, 1049, 420], [15553, -1587, 585, 38, 427, 9], [-9004, -763, 175, 458, 133, 378], [24770, -291, 213, 70, 383, 704], [-1077, -1601, -121, 253, 399, -73], [22433, -2290, 420, -10, 847, -278], [-7025, -1406, 106, 179, -1662, 105], [30, -1010, 368, 84, 318, 348], [-34785, 2178, 290, -214, -3, -566], [4169, -178, -207, -619, -272, 157], [-23760, -510, -238, 6, 186, 144], [-11272, -2075, -183, -147, 1017, -230], [10231, 921, 48, 10, 534, 726], [22942, 146, -372, 114, -1785, -175], [-16291, 531, -202, -115, -1223, -25], [60234, 961, 51, 259, -229, -74], [6533, -1405, 12, -99, 413, -619], [58532, -1226, -171, 448, -414, -504], [-44962, 254, -542, 500, 545, 403], [-8744, 2253, 80, -54, 955, 672], [-22185, 882, 187, -148, 1118, 483], [31841, -1056, -334, -133, -235, -64], [1526, -202, 584, -146, -62, 710], [-7928, -505, -635, -101, 1105, -108], [-47181, 198, -388, 35, 688, 267], [5212, 1233, -331, -107, 598, -485], [34354, 3960, -108, -154, 360, -236], [11318, -444, -105, 21, -1106, -92], [-22458, 1142, 414, -132, -162, 386], [1799, 908, 4, 423, 444, 289], [18157, 558, 276, 217, -1117, -680], [-81444, -72, 220, -338, -1160, 106], [9662, -405, 24, 458, -875, -708], [-35831, 952, 8, 161, -840, -355], [-42663, -426, 606, -640, 764, 727], [20175, 1968, -489, -62, 1410, 39], [-43905, -1868, 220, 306, -1355, 160], [-4278, 439, -415, -185, 585, 754], [31911, 1509, -613, -328, 37, 529], [-38271, 891, -925, 519, 375, 175], [-6182, 3379, 97, 138, -855, -382], [-23122, 1546, -666, -280, -608, -156], [47581, 1924, -501, 297, -990, -103], [-30320, 941, -739, 64, -923, -438], [18664, 2156, 158, 451, -192, 308], [-43790, 892, -270, 308, -201, -501], [10332, 1970, 82, 486, -142, -490], [56334, -1062, -177, 172, -906, -442], [-22350, 3888, -4, -80, -504, 240], [83914, -763, 3, -135, 673, -288], [-59670, -697, 195, -121, -707, -192], [57336, -1347, 87, 145, 176, -215], [122, 998, 210, 67, 896, 164], [16719, -2488, 44, 298, 971, 706], [-14241, -844, -530, -126, -977, 378], [-50267, -163, -30, -69, -915, 57], [-49940, 58, -570, -250, -1060, 228], [-7144, 876, -450, 122, 608, 132], [48279, -965, -208, -101, 439, -297], [-61890, 1393, 298, -380, -151, -170], [16801, 413, -342, -312, 1020, -594], [-10097, 1775, 451, 66, 1333, 439], [93505, 2324, -318, 156, -386, -273], [2479, 3240, 457, -734, 1043, -222], [-44011, 1098, 392, -388, -383, 36], [-46756, -2418, 689, 24, -40, -614], [25801, -1504, 326, -39, -366, 397], [-25051, 3277, -236, -559, 87, 263], [-2067, -328, -1452, 242, 1618, 479], [-28602, -2514, -278, -188, -1052, -56], [71362, -649, -335, -23, 92, -283], [-9121, -3861, 230, 147, 738, 942], [-21294, 2461, 95, 397, 1428, 903], [-3315, -2166, 1105, -168, 314, 145], [-64141, 4350, -288, -137, -594, -191], [-5222, 1017, 511, -263, -1825, -786], [63532, -1424, 368, -62, -1772, -824], [-61394, 2491, -1161, 558, 545, 540], [3071, -3249, 984, -446, -57, 217], [44342, -2001, 321, -158, 641, -216], [-36607, -852, 464, -67, 586, 183], [41879, 2132, -86, -105, -358, -161], [35031, -1450, -63, -12, 482, -147], [51306, 1206, -358, 283, -484, -276], [62843, -4778, 228, -82, -789, -588], [-25801, -2004, 898, -551, -443, 294], [-10009, -1018, -234, 374, 352, -183], [-88683, -2866, 31, 50, 191, -342], [39413, -1528, 487, -291, -1278, -691], [-7966, 2399, -35, 216, -199, 608], [22490, 838, 740, -736, 154, 42], [-38186, -3506, 452, 47, -1926, -110], [1037, 2311, -482, 595, 2451, 157], [66558, 500, -334, 593, -1205, -279], [18555, 120, -592, 263, 711, -8], [49309, -2913, 194, 291, -331, 147], [-49410, -304, -866, 286, 1215, -417], [-2508, -64, 0, 278, -672, 914], [-18016, -3937, 87, -171, -341, -506], [-15398, -2513, -678, 878, -1487, 310], [46841, -3626, 944, -551, 509, -518], [-99523, -536, -128, 271, 1192, 825], [26726, 1240, 654, 932, -558, -550], [7727, -2636, 498, -527, -1650, -721], [100251, -835, 430, -642, -352, -348], [7846, -1458, -422, -217, -2452, 60], [-36927, 2670, 85, 56, -768, 805], [23988, 1304, 246, 190, -2488, -164], [-28818, -2543, 1193, -459, 732, 1035], [97284, 165, 728, -180, -751, -156], [-1237, -5609, 399, -417, -151, 671], [-4160, -1713, -853, 305, -417, -78], [81287, 3690, 872, 470, 177, 754], [44054, -2745, -269, 533, 1641, 1520], [-35577, 278, -115, 87, 144, 243], [-21162, -1480, 506, -96, -844, 592], [28228, -3938, 616, 542, 452, -656], [20, 42, -384, -754, -922, -862], [58303, -1598, -148, -827, 894, -955], [26323, -2639, -926, 223, 1776, 70], [-37895, 512, 164, -265, -892, -501], [10739, -1091, -898, -133, -282, -780], [-132018, -326, 272, -294, -449, -787], [-21094, 6582, 174, 502, -769, -867], [88068, -997, -445, -11, 1071, -766], [39183, 780, 6, 477, 1839, 360], [-113686, -4806, 621, 198, -86, -198], [-82190, 3461, -211, 169, 351, 570], [-13520, -1717, 239, -113, 1417, 16], [-10191, 3122, -14, -196, 615, -850], [15898, 2111, -189, -661, -1207, -452], [46549, 594, -100, 686, 243, 106], [-66868, -5274, 208, -130, 1782, 702], [32028, -396, -230, 456, -1351, 319], [38488, 5201, 285, 85, 593, -218], [-66528, 3097, -851, -186, 425, 638], [-8928, 2560, 288, -130, -340, 682], [7645, -5129, -522, -57, -167, 615], [-6179, 2983, -804, 212, 915, 1155], [-43081, -2909, 124, 390, 746, 560], [96541, -1999, -499, 48, -585, -927], [85916, 1075, 3, -764, 225, -990], [34676, -31, -79, -842, -991, -1282], [-39100, 1254, 1380, -1262, 866, -838], [81625, 1150, 146, 489, 660, 205], [-96655, 3838, -694, 848, -325, 90], [28795, -1726, 408, -1086, -1289, 912], [-73720, 2072, 794, -132, -2124, 638], [86577, 2354, 268, 122, 1146, 217], [-4160, -3815, -1, -700, -561, -1564], [9184, -2667, 1063, -40, 1359, -712], [-15300, 1605, -1455, 401, 2455, -120], [93436, 204, 1587, -63, 1654, -98], [-11600, 1866, 732, -405, -1617, -759], [59232, 4383, -485, -177, 135, -96], [-75149, -1384, 164, 79, -1136, -2027], [-37826, -1996, 263, 462, 1590, 696], [544, -6266, -387, 447, 460, 742], [46642, -4217, 1043, -524, -831, -528], [-97998, -9, -223, 97, 1628, -431], [148773, 1101, -782, 1, 481, -809], [-27381, 2527, -68, 75, 1495, -705], [90770, 2842, -246, 492, -1426, -424], [-69628, -3213, 875, 556, -217, -354], [51453, 5631, 383, 613, -2061, -223], [58751, 2720, -487, -234, 2579, 1458], [29278, -7086, 178, 202, 664, 668]]
